use rocket::{
    form::Form,
    response::{Flash, Redirect},
    Route, State,
};
use sea_orm::{ActiveModelTrait, ColumnTrait, DatabaseConnection, EntityTrait, QueryFilter, Set};

use crate::models::{day, trip, user};

use super::NaiveDateForm;

#[derive(FromForm)]
struct RegisterForm {
    day: NaiveDateForm,
    #[field(validate = len(3..))]
    name: String,
    time: Option<String>,
    cox_id: Option<i32>,
}

#[put("/", data = "<register>")]
async fn register(
    db: &State<DatabaseConnection>,
    register: Form<RegisterForm>,
    user: user::Model,
) -> Flash<Redirect> {
    let day = day::Entity::find_by_id(*register.day)
        .one(db.inner())
        .await
        .unwrap()
        .expect("There's no trip on this date (yet)");

    if !(day.open_registration || user.is_cox) {
        log::error!("{} tried to register, even though the user it should not be possible to do so via UI -> manually crafted request?", user.name);
        return Flash::error(
            Redirect::to("/"),
            "Don't (try to ;)) abuse this system! Incident has been reported...",
        );
    }

    let user = user::Model::find_or_create_user(&register.name, db.inner()).await;

    let day = format!("{}", day.day.format("%Y-%m-%d"));
    let trip = trip::ActiveModel {
        day: Set(day.clone()),
        user_id: Set(user.id),
        begin: Set(register.time.clone()),
        cox_id: Set(register.cox_id.clone()),
        ..Default::default()
    };

    match trip.insert(db.inner()).await {
        Ok(_) => {
            log::info!("{} registered for {:?}", user.name, day);
            Flash::success(Redirect::to("/"), "Erfolgreich angemeldet!")
        }
        Err(_) => {
            log::warn!(
                "{} tried to register for {:?}, but is already registered",
                user.name,
                day
            );
            Flash::error(Redirect::to("/"), "Du bist bereits angemeldet")
        }
    }
}

#[derive(FromForm)]
struct DeleteForm {
    day: NaiveDateForm,
    user: i32,
    cox_id: Option<i32>,
    begin: Option<String>,
}

#[delete("/", data = "<delete>")]
async fn delete(
    db: &State<DatabaseConnection>,
    delete: Form<DeleteForm>,
    user: user::Model,
) -> Flash<Redirect> {
    let day = day::Entity::find_by_id(*delete.day)
        .one(db.inner())
        .await
        .unwrap()
        .expect("There's no trip on this date (yet)");

    if delete.user != user.id {
        log::error!("{} tried to delete a registration from user_id {} on day {:?} (probably hand-crafted request)", user.name, delete.user, delete.day);
        return Flash::error(
            Redirect::to("/"),
            "Du kannst nur deine eigenen Anmeldungen löschen!",
        );
    }

    let trip = trip::Entity::find()
        .filter(trip::Column::Day.eq(format!("{}", day.day.format("%Y-%m-%d"))))
        .filter(trip::Column::UserId.eq(user.id))
        .filter(trip::Column::CoxId.eq(delete.cox_id))
        .filter(trip::Column::Begin.eq(delete.begin.clone()))
        .one(db.inner())
        .await
        .unwrap();
    match trip {
        None => {
            log::error!("Tried to register w/o being authenticated (prob. hand crafted request (user.name = {})", user.name);
            return Flash::error(Redirect::to("/"), "Du bist gar nicht angemeldet!");
        }
        Some(trip) => {
            log::info!("User {} deleted the registration for {:?}", user.name, trip);
            trip::Entity::delete(trip::ActiveModel {
                day: Set(trip.day),
                user_id: Set(trip.user_id),
                //cox_id: Set(delete.cox_id),
                begin: Set(delete.begin.clone()),
                ..Default::default()
            })
            .exec(db.inner())
            .await
            .unwrap();
        }
    }

    Flash::success(Redirect::to("/"), "Abmeldung erfolgreich")
}
pub fn routes() -> Vec<Route> {
    routes![register, delete]
}
